#!/usr/bin/env python
"""
Modelica parse Tree to AST tree.
"""
from __future__ import print_function, absolute_import, division, print_function, unicode_literals

import os
import sys
import time
import unittest

from pymola import parser
from pymola import tree

TEST_DIR = os.path.dirname(os.path.realpath(__file__))


class ParseTest(unittest.TestCase):
    """
    Parse test
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @staticmethod
    def flush():
        sys.stdout.flush()
        sys.stdout.flush()
        time.sleep(0.1)

    def test_aircraft(self):
        with open(os.path.join(TEST_DIR, 'Aircraft.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        print('AST TREE\n', ast_tree)
        flat_tree = tree.flatten(ast_tree, 'Aircraft')
        print('AST TREE FLAT\n', flat_tree)
        self.flush()

    @unittest.skip
    def test_bouncing_ball(self):
        with open(os.path.join(TEST_DIR, 'BouncingBall.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        print('AST TREE\n', ast_tree)
        flat_tree = tree.flatten(ast_tree, 'BouncingBall')
        print(flat_tree)
        print('AST TREE FLAT\n', flat_tree)
        self.flush()

    def test_estimator(self):
        with open(os.path.join(TEST_DIR, './Estimator.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        print('AST TREE\n', ast_tree)
        flat_tree = tree.flatten(ast_tree, 'Estimator')
        print('AST TREE FLAT\n', flat_tree)
        self.flush()

    def test_spring(self):
        with open(os.path.join(TEST_DIR, 'Spring.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        print('AST TREE\n', ast_tree)
        flat_tree = tree.flatten(ast_tree, 'Spring')
        print('AST TREE FLAT\n', flat_tree)
        self.flush()

    def test_duplicate_state(self):
        with open(os.path.join(TEST_DIR, 'DuplicateState.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        print('AST TREE\n', ast_tree)
        flat_tree = tree.flatten(ast_tree, 'DuplicateState')
        print('AST TREE FLAT\n', flat_tree)
        self.flush()

    def test_connector(self):
        with open(os.path.join(TEST_DIR, 'Connector.mo'), 'r') as f:
            txt = f.read()
        # noinspection PyUnusedLocal
        ast_tree = parser.parse(txt)
        # states = ast_tree.classes['Aircraft'].states
        # names = sorted([state.name for state in states])
        # names_set = sorted(list(set(names)))
        # if names != names_set:
        #     raise IOError('{:s} != {:s}'.format(str(names), str(names_set)))
        self.flush()

    def test_inheritance(self):
        with open(os.path.join(TEST_DIR, 'InheritanceInstantiation.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        flat_tree = tree.flatten(ast_tree, 'C2')

        self.assertEqual(flat_tree.classes['C2'].symbols['bcomp.b'].value.value, 3.0)

    # TODO: Currently nested (local) classes are not parsed correctly. Not
    # entirely sure if their scope is local, or other (non-extending) classes
    # can make instantiations as well. That will likely influence the way they
    # need to be stored.
    @unittest.skip
    def test_inheritance(self):
        with open(os.path.join(TEST_DIR, 'InheritanceInstantiation.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        flat_tree = tree.flatten(ast_tree, 'C2')

        self.assertEqual(flat_tree.classes['C2'].symbols['bcomp.b'].value.value, 3.0)

    def test_inheritance_symbol_modifiers(self):
        with open(os.path.join(TEST_DIR, 'Inheritance.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        flat_tree = tree.flatten(ast_tree, 'Sub')

        self.assertEqual(flat_tree.classes['Sub'].symbols['x'].max.value, 30.0)

if __name__ == "__main__":
    unittest.main()



from __future__ import print_function, absolute_import, division, unicode_literals

import itertools
import logging
from collections import namedtuple

import casadi as ca
import numpy as np
from typing import Union

from . import ast
from .tree import TreeWalker, TreeListener, flatten

logger = logging.getLogger("pymola")

# TODO
#  - Nested for loops

OP_MAP = {'*': "__mul__",
          '+': "__add__",
          "-": "__sub__",
          "/": "__div__",
          '>': '__gt__',
          '<': '__lt__',
          '<=': '__le__',
          '>=': '__ge__',
          '!=': '__ne__',
          '==': '__eq__',
          "min": "fmin",
          "max": "fmax",
          "abs": "fabs"}

VariableMetadata = namedtuple('VariableMetadata', ['min', 'max', 'nominal'])


# noinspection PyUnresolvedReferences
class CasadiSysModel:
    def __init__(self):
        self.states = []
        self.state_metadata = []
        self.der_states = []
        self.alg_states = []
        self.alg_state_metadata = []
        self.inputs = []
        self.outputs = []
        self.constants = []
        self.constant_values = []
        self.parameters = []
        self.parameter_values = []
        self.equations = []
        self.time = ca.MX.sym('time')
        self.delayed_states = []

    def __str__(self):
        r = ""
        r += "Model\n"
        r += "time: " + str(self.time) + "\n"
        r += "states: " + str(self.states) + "\n"
        r += "der_states: " + str(self.der_states) + "\n"
        r += "alg_states: " + str(self.alg_states) + "\n"
        r += "inputs: " + str(self.inputs) + "\n"
        r += "outputs: " + str(self.outputs) + "\n"
        r += "constants: " + str(self.constants) + "\n"
        r += "constant_values: " + str(self.constant_values) + "\n"
        r += "parameters: " + str(self.parameters) + "\n"
        r += "equations: " + str(self.equations) + "\n"
        return r

    def check_balanced(self):
        n_states = sum(v.size1() * v.size2() for v in itertools.chain(self.states, self.alg_states))
        n_inputs = sum(v.size1() * v.size2() for v in self.inputs)
        n_equations = sum(e.size1() * e.size2() for e in self.equations)
        if n_states - n_inputs == n_equations:
            logger.info("System is balanced.")
        else:
            logger.warning(
                "System is not balanced.  "
                "Number of states minus inputs is {}, number of equations is {}.".format(
                    n_states - n_inputs, n_equations))

    def simplify(self, replace_constants=True, replace_parameter_expressions=True):
        if replace_constants:
            self.equations = ca.substitute(self.equations, self.constants, self.constant_values)
            self.constants = []
            self.constant_values = []

        if replace_parameter_expressions:
            composite_parameters, simple_parameters = [], []
            composite_parameter_values, simple_parameter_values = [], []
            for e, v in zip(self.parameters, self.parameter_values):
                is_composite = isinstance(v, ca.MX) and not v.is_constant()
                (simple_parameters, composite_parameters)[is_composite].append(e)
                (simple_parameter_values, composite_parameter_values)[is_composite].append(
                    float(v) if not is_composite and isinstance(v, ca.MX) else v)

            self.equations = ca.substitute(self.equations, composite_parameters, composite_parameter_values)
            self.parameters = simple_parameters
            self.parameter_values = simple_parameter_values

            # TODO detect and eliminate aliases
            # TODO eliminate protected variables without min/max attribute

    def dae_residual_function(self, group_arguments=True):
        if group_arguments:
            return ca.Function('dae_residual', [self.time, ca.vertcat(*self.states), ca.vertcat(*self.der_states),
                                                ca.vertcat(*self.alg_states), ca.vertcat(*self.constants),
                                                ca.vertcat(*self.parameters)], [ca.vertcat(*self.equations)])
        else:
            return ca.Function('dae_residual', [
                self.time] + self.states + self.der_states + self.alg_states + self.constants + self.parameters,
                               self.equations)

    # noinspection PyUnusedLocal
    def initial_residual_function(self, group_arguments=True):
        # TODO
        return ca.Function('initial_residual', [self.time], [0])

    # noinspection PyPep8Naming
    def state_metadata_function(self, group_arguments=True):
        m, M, n = [], [], []
        for e, v in zip(itertools.chain(self.states, self.alg_states),
                        itertools.chain(self.state_metadata, self.alg_state_metadata)):
            m_ = v.min if hasattr(v.min, '__iter__') else np.full(e.size(), v.min if v.min is not None else -np.inf)
            M_ = v.max if hasattr(v.max, '__iter__') else np.full(e.size(), v.max if v.max is not None else np.inf)
            n_ = v.nominal if hasattr(v.nominal, '__iter__') else np.full(e.size(),
                                                                          v.nominal if v.nominal is not None else 1)
            m.append(m_)
            M.append(M_)
            n.append(n_)
        out = ca.horzcat(ca.vertcat(*m), ca.vertcat(*M), ca.vertcat(*n))
        if group_arguments:
            return ca.Function('state_metadata', [ca.vertcat(*self.parameters)],
                               [out[:len(self.state_metadata), :], out[len(self.state_metadata):, :]])
        else:
            return ca.Function('state_metadata', self.parameters, [out])


ForLoopIndexedSymbol = namedtuple('ForLoopSymbol', ['tree', 'indices'])


# noinspection PyPep8Naming,PyUnresolvedReferences
class ForLoop:
    def __init__(self, generator, tree):
        self.tree = tree
        self.generator = generator
        i = tree.indices[0]
        e = i.expression
        start = e.start.value
        step = e.step.value
        stop = self.generator.get_integer(e.stop)
        self.values = np.arange(start, stop + step, step, dtype=np.int)
        self.index_variable = ca.MX.sym(i.name)
        self.name = i.name
        self.indexed_symbols = {}

    def register_indexed_symbol(self, e, tree, index_expr=None):
        if isinstance(index_expr, ca.MX):
            F = ca.Function('index_expr', [self.index_variable], [index_expr])
            # expr = lambda ar: np.array([F(a)[0] for a in ar], dtype=np.int)
            Fmap = F.map("map", "serial", len(self.values), [], [])
            res = Fmap.call([self.values])
            indices = np.array(res[0].T, dtype=np.int)
        else:
            indices = self.values
        self.indexed_symbols[e] = ForLoopIndexedSymbol(tree, indices)


# noinspection PyPep8Naming,PyUnresolvedReferences
class CasadiGenerator(TreeListener):
    def __init__(self, root, class_name):
        super(CasadiGenerator, self).__init__()
        self.src = {}
        self.model = CasadiSysModel()
        self.nodes = {'time': self.model.time}
        self.derivative = {}
        self.root = root
        self.class_name = class_name
        self.for_loops = []

    def exitClass(self, tree):
        logger.debug('exitClass {}'.format(tree.name))

        states = []
        inputs = []
        outputs = []
        constants = []
        parameters = []
        symbols = sorted(tree.symbols.values(), key=lambda x: x.order)
        for s in symbols:
            if 'constant' in s.prefixes:
                constants.append(s)
            elif 'parameter' in s.prefixes:
                parameters.append(s)
            else:
                states.append(s)
                if 'input' in s.prefixes:
                    inputs.append(s)
                elif 'output' in s.prefixes:
                    outputs.append(s)

        def discard_empty(l):
            return list(filter(lambda x: not x.is_empty(), l))

        ode_states = []
        alg_states = []
        for s in states:
            if self.get_mx(s) in self.derivative:
                ode_states.append(s)
            else:
                alg_states.append(s)
        self.model.states = discard_empty([self.get_mx(e) for e in ode_states])
        self.model.state_metadata = [VariableMetadata(self.get_mx(e.min), self.get_mx(e.max), self.get_mx(e.nominal))
                                     for e in ode_states if not self.get_mx(e).is_empty()]
        self.model.der_states = discard_empty([self.derivative[
                                                   self.get_mx(e)] for e in ode_states])
        self.model.alg_states = discard_empty([self.get_mx(e) for e in alg_states])
        self.model.alg_state_metadata = [
            VariableMetadata(self.get_mx(e.min), self.get_mx(e.max), self.get_mx(e.nominal)) for e in alg_states if
            not self.get_mx(e).is_empty()]
        assert len(self.model.alg_states) == len(self.model.alg_state_metadata)
        self.model.constants = discard_empty([self.get_mx(e) for e in constants])
        self.model.constant_values = [self.get_mx(e.value) for e in constants if not self.get_mx(e).is_empty()]
        self.model.parameters = discard_empty([self.get_mx(e) for e in parameters])
        self.model.parameter_values = [self.get_mx(e.value) for e in parameters if not self.get_mx(e).is_empty()]
        self.model.inputs = discard_empty([self.get_mx(e) for e in inputs])
        self.model.outputs = discard_empty([self.get_mx(e) for e in outputs])
        self.model.equations = discard_empty([self.get_mx(e) for e in tree.equations])

    def exitArray(self, tree):
        self.src[tree] = [self.src[e] for e in tree.values]

    def exitPrimary(self, tree):
        self.src[tree] = tree.value

    def exitExpression(self, tree):
        if isinstance(tree.operator, ast.ComponentRef):
            op = tree.operator.name
        else:
            op = tree.operator

        if op == '*':
            op = 'mtimes'  # .* differs from *
        if op.startswith('.'):
            op = op[1:]

        logger.debug('exitExpression')

        n_operands = len(tree.operands)
        if op == 'der':
            orig = self.get_mx(tree.operands[0])
            if orig in self.derivative:
                src = self.derivative[orig]
            else:
                s = ca.MX.sym("der({})".format(orig.name()), orig.sparsity())
                self.derivative[orig] = s
                self.nodes[s] = s
                src = s
        elif op == '-' and n_operands == 1:
            src = -self.get_mx(tree.operands[0])
        elif op == 'mtimes':
            src = self.get_mx(tree.operands[0])
            for i in tree.operands[1:]:
                src = ca.mtimes(src, self.get_mx(i))
        elif op == 'transpose' and n_operands == 1:
            src = self.get_mx(tree.operands[0]).T
        elif op == 'sum' and n_operands == 1:
            v = self.get_mx(tree.operands[0])
            src = ca.sum1(v)
        elif op == 'linspace' and n_operands == 3:
            a = self.get_mx(tree.operands[0])
            b = self.get_mx(tree.operands[1])
            n_steps = self.get_integer(tree.operands[2])
            src = ca.linspace(a, b, n_steps)
        elif op == 'fill' and n_operands == 2:
            val = self.get_mx(tree.operands[0])
            n_row = self.get_integer(tree.operands[1])
            src = val * ca.DM.ones(n_row)
        elif op == 'fill' and n_operands == 3:
            val = self.get_mx(tree.operands[0])
            n_row = self.get_integer(tree.operands[1])
            n_col = self.get_integer(tree.operands[2])
            src = val * ca.DM.ones(n_row, n_col)
        elif op == 'zeros' and n_operands == 1:
            n_row = self.get_integer(tree.operands[0])
            src = ca.DM.zeros(n_row)
        elif op == 'zeros' and n_operands == 2:
            n_row = self.get_integer(tree.operands[0])
            n_col = self.get_integer(tree.operands[1])
            src = ca.DM.zeros(n_row, n_col)
        elif op == 'ones' and n_operands == 1:
            n_row = self.get_integer(tree.operands[0])
            src = ca.DM.ones(n_row)
        elif op == 'ones' and n_operands == 2:
            n_row = self.get_integer(tree.operands[0])
            n_col = self.get_integer(tree.operands[1])
            src = ca.DM.ones(n_row, n_col)
        elif op == 'identity' and n_operands == 1:
            n = self.get_integer(tree.operands[0])
            src = ca.DM.eye(n)
        elif op == 'diagonal' and n_operands == 1:
            diag = self.get_mx(tree.operands[0])
            n = len(diag)
            indices = list(range(n))
            src = ca.DM.triplet(indices, indices, diag, n, n)
        elif op == 'delay' and n_operands == 2:
            expr = self.get_mx(tree.operands[0])
            delay_time = self.get_mx(tree.operands[1])
            assert isinstance(expr, MX)
            src = ca.MX.sym('{}_delayed_{}'.format(
                expr.name(), delay_time), expr.size1(), expr.size2())
            self.model.delayed_states.append((src, expr, delay_time))
        elif op in OP_MAP and n_operands == 2:
            lhs = self.get_mx(tree.operands[0])
            rhs = self.get_mx(tree.operands[1])
            lhs_op = getattr(lhs, OP_MAP[op])
            src = lhs_op(rhs)
        elif op in OP_MAP and n_operands == 1:
            lhs = self.get_mx(tree.operands[0])
            lhs_op = getattr(lhs, OP_MAP[op])
            src = lhs_op()
        elif n_operands == 1:
            src = self.get_mx(tree.operands[0])
            src = getattr(src, tree.operator.name)()
        elif n_operands == 2:
            lhs = self.get_mx(tree.operands[0])
            rhs = self.get_mx(tree.operands[1])
            lhs_op = getattr(lhs, tree.operator.name)
            src = lhs_op(rhs)
        else:
            raise Exception("Unknown operator {}({})".format(op, ','.join(n_operands * ['.'])))
        self.src[tree] = src

    def exitIfExpression(self, tree):
        logger.debug('exitIfExpression')

        assert (len(tree.conditions) + 1 == len(tree.expressions))

        src = self.get_mx(tree.expressions[-1])
        for cond_index in range(len(tree.conditions)):
            cond = self.get_mx(tree.conditions[-(cond_index + 1)])
            expr1 = self.get_mx(tree.expressions[-(cond_index + 2)])

            src = ca.if_else(cond, expr1, src)

        self.src[tree] = src

    def exitEquation(self, tree):
        logger.debug('exitEquation')

        self.src[tree] = self.get_mx(tree.left) - self.get_mx(tree.right)

    def enterForEquation(self, tree):
        logger.debug('enterForEquation')

        self.for_loops.append(ForLoop(self, tree))

    def exitForEquation(self, tree):
        logger.debug('exitForEquation')

        f = self.for_loops.pop()
        if len(f.values) > 0:
            indexed_symbols = list(f.indexed_symbols.keys())
            args = [f.index_variable] + indexed_symbols
            expr = ca.vcat([ca.vec(self.get_mx(e)) for e in tree.equations])
            free_vars = ca.symvar(expr)

            arg_names = [arg.name() for arg in args]
            free_vars = [e for e in free_vars if e.name() not in arg_names]
            all_args = args + free_vars
            F = ca.Function('loop_body_' + f.name, all_args, [expr])

            indexed_symbols_full = [self.nodes[
                                        f.indexed_symbols[k].tree.name][f.indexed_symbols[k].indices - 1] for k in
                                    indexed_symbols]
            Fmap = F.map("map", "serial", len(f.values), list(
                range(len(args), len(all_args))), [])
            res = Fmap.call([f.values] + indexed_symbols_full + free_vars)

            self.src[tree] = res[0].T
        else:
            self.src[tree] = ca.MX()

    def exitIfEquation(self, tree):
        logger.debug('exitIfEquation')

        assert (len(tree.equations) % (len(tree.conditions) + 1) == 0)

        equations_per_condition = int(
            len(tree.equations) / (len(tree.conditions) + 1))

        src = ca.vertcat(*[self.get_mx(tree.equations[-(i + 1)])
                           for i in range(equations_per_condition)])
        for cond_index in range(len(tree.conditions)):
            cond = self.get_mx(tree.conditions[-(cond_index + 1)])
            expr1 = ca.vertcat(*[self.get_mx(tree.equations[-equations_per_condition * (
                cond_index + 1) - (i + 1)]) for i in range(equations_per_condition)])

            src = ca.if_else(cond, expr1, src)

        self.src[tree] = src

    def get_integer(self, tree: Union[ast.Primary, ast.ComponentRef, ast.Expression, ast.Slice]):
        # CasADi needs to know the dimensions of symbols at instantiation.
        # We therefore need a mechanism to evaluate expressions that define dimensions of symbols.
        if isinstance(tree, ast.Primary):
            return int(tree.value)
        if isinstance(tree, ast.ComponentRef):
            s = self.root.find_symbol(self.root.classes[self.class_name], tree)
            assert (s.type.name == 'Integer')
            return self.get_integer(s.value)
        if isinstance(tree, ast.Expression):
            # Make sure that the expression has been converted to MX by (re)visiting the
            # relevant part of the AST.
            ast_walker = TreeWalker()
            ast_walker.walk(self, tree)

            # Obtain expression

            expr = self.get_mx(tree)

            # Obtain the symbols it depends on
            deps = [expr.dep(i) for i in range(expr.n_dep())
                    if expr.dep(i).is_symbolic()]

            # Find the values of the symbols
            vals = []
            for dep in deps:
                if dep.is_symbolic():
                    if (len(self.for_loops) > 0) and (dep.name() == self.for_loops[-1].name):
                        vals.append(self.for_loops[-1].index_variable)
                    else:
                        vals.append(self.get_integer(self.root.find_symbol(self.root.classes[self.class_name],
                                                                           ast.ComponentRef(name=dep.name())).value))

            # Evaluate the expression
            F = ca.Function('get_integer_{}'.format('_'.join([dep.name().replace('.', '_') for dep in deps])), deps,
                            [expr])
            ret = F.call(vals)
            if ret[0].is_constant():
                return int(ret[0])
            else:
                return ret[0]
        if isinstance(tree, ast.Slice):
            start = self.get_integer(tree.start)
            step = self.get_integer(tree.step)
            stop = self.get_integer(tree.stop)
            return np.arange(start, stop + step, step, dtype=np.int)
        else:
            raise Exception('Unexpected node type {}'.format(tree.__class__.__name__))

    def get_symbol(self, tree):
        # Create symbol
        size = [self.get_integer(d) for d in tree.dimensions]
        assert (len(size) <= 2)
        s = ca.MX.sym(tree.name, *size)
        self.nodes[tree.name] = s
        return s

    def get_indexed_symbol(self, tree, s):
        # Check whether we loop over an index of this symbol
        indices = []
        for index in tree.indices:
            if isinstance(index, ast.ComponentRef):
                for for_loop in self.for_loops:
                    if index.name == for_loop.name:
                        # TODO support nested loops
                        s = ca.MX.sym('{}[{}]'.format(tree.name, for_loop.name))
                        for_loop.register_indexed_symbol(s, tree)
                        return s

            sl = self.get_integer(index)
            if not isinstance(sl, int) and not isinstance(sl, np.ndarray):
                for_loop = self.for_loops[-1]
                s = ca.MX.sym('{}[{}]'.format(tree.name, for_loop.name))
                for_loop.register_indexed_symbol(s, tree, sl)
                return s

            # Modelica indexing starts from one;  Python from zero.
            indices.append(sl - 1)
        if len(indices) == 1:
            return s[indices[0]]
        elif len(indices) == 2:
            return s[indices[0], indices[1]]
        else:
            raise Exception("Dimensions higher than two are not yet supported")

    def get_component(self, tree):
        # Check special symbols
        if tree.name == 'time':
            return self.model.time
        else:
            for f in reversed(self.for_loops):
                if f.name == tree.name:
                    return f.index_variable

        # Check ordinary symbols
        symbol = self.root.find_symbol(self.root.classes[self.class_name], tree)
        s = self.get_mx(symbol)
        if len(tree.indices) > 0:
            s = self.get_indexed_symbol(tree, s)
        return s

    def get_mx(self, tree: Union[ast.Symbol, ast.ComponentRef, ast.Expression]) -> ca.MX:
        """
        We pull components and symbols from the AST on demand.  
        This is to ensure that parametrized vector dimensions can be resolved.  Vector
        dimensions need to be known at CasADi MX creation time.
        :param tree: 
        :return: 
        """
        if tree not in self.src:
            if isinstance(tree, ast.Symbol):
                s = self.get_symbol(tree)
            elif isinstance(tree, ast.ComponentRef):
                s = self.get_component(tree)
            else:
                raise Exception('Tried to look up expression before it was reached by the tree walker')
            self.src[tree] = s
        return self.src[tree]


def generate(ast_tree, model_name):
    # create a walker
    ast_walker = TreeWalker()

    flat_tree = flatten(ast_tree, model_name)

    casadi_gen = CasadiGenerator(flat_tree, model_name)
    ast_walker.walk(casadi_gen, flat_tree)
    return casadi_gen.model




# do not edit, generated by pymola

from __future__ import print_function, division
import sympy
import sympy.physics.mechanics as mech
from pymola.sympy_runtime import OdeModel
from sympy import sin, cos, tan


class Spring(OdeModel):

    def __init__(self):

        super(Spring, self).__init__()

        # states
        x, v_x = mech.dynamicsymbols('x, v_x')
        self.x = sympy.Matrix([x, v_x])
        self.x0 = {
            x : 0,
            v_x : 0,
            }

        # variables
        self.v = sympy.Matrix([])

        # constants
        self.c = sympy.Matrix([])
        self.c0 = {
            }

        # parameters
        c, k = sympy.symbols('c, k')
        self.p = sympy.Matrix([c, k])
        self.p0 = {
            c : 0,
            k : 0,
            }

        # inputs
        self.u = sympy.Matrix([])
        self.u0 = {
            }

        # outputs
        self.y = sympy.Matrix([])

        # equations
        self.eqs = [
            (x).diff(self.t) - (v_x),
            (v_x).diff(self.t) - (- k * x - c * v_x),
            ]

        self.compute_fg()


#!/usr/bin/env python
"""
Tools for tree walking and visiting etc.
"""

from __future__ import print_function, absolute_import, division, unicode_literals

import copy
import logging
import sys
from collections import OrderedDict

from typing import Union

from . import ast

CLASS_SEPARATOR = '.'

logger = logging.getLogger("pymola")


# TODO Flatten function vs. conversion classes


# noinspection PyPep8Naming
class TreeListener(object):
    """
    Defines interface for tree listeners.
    """

    def __init__(self):
        self.context = {}

    def enterEvery(self, tree: ast.Node) -> None:
        self.context[type(tree).__name__] = tree

    def exitEvery(self, tree: ast.Node):
        self.context[type(tree).__name__] = None

    def enterFile(self, tree: ast.File) -> None:
        pass

    def exitFile(self, tree: ast.File) -> None:
        pass

    def enterClass(self, tree: ast.Class) -> None:
        pass

    def exitClass(self, tree: ast.Class) -> None:
        pass

    def enterImportAsClause(self, tree: ast.ImportAsClause) -> None:
        pass

    def exitImportAsClause(self, tree: ast.ImportAsClause) -> None:
        pass

    def enterImportFromClause(self, tree: ast.ImportFromClause) -> None:
        pass

    def exitImportFromClause(self, tree: ast.ImportFromClause) -> None:
        pass

    def enterElementModification(self, tree: ast.ElementModification) -> None:
        pass

    def exitElementModification(self, tree: ast.ElementModification) -> None:
        pass

    def enterClassModification(self, tree: ast.ClassModification) -> None:
        pass

    def exitClassModification(self, tree: ast.ClassModification) -> None:
        pass

    def enterExtendsClause(self, tree: ast.ExtendsClause) -> None:
        pass

    def exitExtendsClause(self, tree: ast.ExtendsClause) -> None:
        pass

    def enterIfExpression(self, tree: ast.IfExpression) -> None:
        pass

    def exitIfExpression(self, tree: ast.IfExpression) -> None:
        pass

    def enterExpression(self, tree: ast.Expression) -> None:
        pass

    def exitExpression(self, tree: ast.Expression) -> None:
        pass

    def enterIfEquation(self, tree: ast.IfEquation) -> None:
        pass

    def exitIfEquation(self, tree: ast.IfEquation) -> None:
        pass

    def enterForIndex(self, tree: ast.ForIndex) -> None:
        pass

    def exitForIndex(self, tree: ast.ForIndex) -> None:
        pass

    def enterForEquation(self, tree: ast.ForEquation) -> None:
        pass

    def exitForEquation(self, tree: ast.ForEquation) -> None:
        pass

    def enterEquation(self, tree: ast.Equation) -> None:
        pass

    def exitEquation(self, tree: ast.Equation) -> None:
        pass

    def enterConnectClause(self, tree: ast.ConnectClause) -> None:
        pass

    def exitConnectClause(self, tree: ast.ConnectClause) -> None:
        pass

    def enterSymbol(self, tree: ast.Symbol) -> None:
        pass

    def exitSymbol(self, tree: ast.Symbol) -> None:
        pass

    def enterComponentClause(self, tree: ast.ComponentClause) -> None:
        pass

    def exitComponentClause(self, tree: ast.ComponentClause) -> None:
        pass

    def enterArray(self, tree: ast.Array) -> None:
        pass

    def exitArray(self, tree: ast.Array) -> None:
        pass

    def enterSlice(self, tree: ast.Slice) -> None:
        pass

    def exitSlice(self, tree: ast.Slice) -> None:
        pass

    def enterPrimary(self, tree: ast.Primary) -> None:
        pass

    def exitPrimary(self, tree: ast.Primary) -> None:
        pass

    def enterComponentRef(self, tree: ast.ComponentRef) -> None:
        pass

    def exitComponentRef(self, tree: ast.ComponentRef) -> None:
        pass


class TreeWalker(object):
    """
    Defines methods for tree walker. Inherit from this to make your own.
    """

    def walk(self, listener: TreeListener, tree: ast.Node) -> None:
        """
        Walks an AST tree recursively
        :param listener: 
        :param tree: 
        :return: None
        """
        name = tree.__class__.__name__
        if hasattr(listener, 'enterEvery'):
            getattr(listener, 'enterEvery')(tree)
        if hasattr(listener, 'enter' + name):
            getattr(listener, 'enter' + name)(tree)
        for child_name in tree.__dict__.keys():
            self.handle_walk(listener, tree.__dict__[child_name])
        if hasattr(listener, 'exitEvery'):
            getattr(listener, 'exitEvery')(tree)
        if hasattr(listener, 'exit' + name):
            getattr(listener, 'exit' + name)(tree)

    def handle_walk(self, listener: TreeListener, tree: Union[ast.Node, dict, list]) -> None:
        """
        Handles tree walking, has to account for dictionaries and lists
        :param listener: listener that reacts to walked events
        :param tree: the tree to walk
        :return: None
        """
        if isinstance(tree, ast.Node):
            self.walk(listener, tree)
        elif isinstance(tree, dict):
            for k in tree.keys():
                self.handle_walk(listener, tree[k])
        elif isinstance(tree, list):
            for i in range(len(tree)):
                self.handle_walk(listener, tree[i])
        else:
            pass


def flatten_class(root: ast.Collection, orig_class: ast.Class, instance_name: str,
                  class_modification: ast.ClassModification = None) -> ast.Class:
    """
    This function takes and flattens it so that all subclasses instances
    are replaced by the their equations and symbols with name mangling
    of the instance name passed.
    :param root: The root of the tree that contains all class definitions
    :param orig_class: The class we want to flatten
    :param instance_name: 
    :param class_modification: 
    :return: flat_class, the flattened class of type Class
    """

    # create the returned class
    flat_class = ast.Class(
        name=orig_class.name,
    )

    # append period to non empty instance_name
    if instance_name != '':
        instance_prefix = instance_name + CLASS_SEPARATOR
    else:
        instance_prefix = instance_name

    extended_orig_class = ast.Class(
        name=orig_class.name,
    )

    for extends in orig_class.extends:
        c = root.find_class(extends.component, orig_class.within)

        # recursively call flatten on the parent class
        # NOTE: We do not to pass the instance name along. The symbol renaming
        # is handled at the current level, not at the level of the base class.
        # That way we can properly apply class modifications to inherited
        # symbols.
        flat_parent_class = flatten_class(root, c, '')

        # set visibility
        for sym in flat_parent_class.symbols.values():
            if sym.visibility > extends.visibility:
                sym.visibility = extends.visibility

        # add parent class members symbols, equations and statements
        extended_orig_class.symbols.update(flat_parent_class.symbols)
        extended_orig_class.equations += flat_parent_class.equations
        extended_orig_class.statements += flat_parent_class.statements

        # carry out modifications
        extended_orig_class = modify_class(root, extended_orig_class, extends.class_modification)

    extended_orig_class.symbols.update(orig_class.symbols)
    extended_orig_class.equations += orig_class.equations
    extended_orig_class.statements += orig_class.statements

    if class_modification is not None:
        extended_orig_class = modify_class(root, extended_orig_class, class_modification)

    # for all symbols in the original class
    for sym_name, sym in extended_orig_class.symbols.items():
        flat_sym = flatten_symbol(sym, instance_prefix)
        try:
            c = root.find_class(flat_sym.type)
        except KeyError:
            # append original symbol to flat class
            flat_class.symbols[flat_sym.name] = flat_sym
        else:
            # recursively call flatten on the contained class
            flat_sub_class = flatten_class(root, c, flat_sym.name, flat_sym.class_modification)

            # carry class dimensions over to symbols
            for flat_class_symbol in flat_sub_class.symbols.values():
                if len(flat_class_symbol.dimensions) == 1 \
                        and isinstance(flat_class_symbol.dimensions[0], ast.Primary) \
                        and flat_class_symbol.dimensions[0].value == 1:
                    flat_class_symbol.dimensions = flat_sym.dimensions
                elif len(flat_sym.dimensions) == 1 and isinstance(flat_sym.dimensions[0], ast.Primary) \
                        and flat_sym.dimensions[0].value == 1:
                    flat_class_symbol.dimensions = flat_class_symbol.dimensions
                else:
                    flat_class_symbol.dimensions = flat_sym.dimensions + flat_class_symbol.dimensions

            # add sub_class members symbols and equations
            flat_class.symbols.update(flat_sub_class.symbols)
            flat_class.equations += flat_sub_class.equations
            flat_class.statements += flat_sub_class.statements

            # we keep connectors in the class hierarchy, as we may refer to them further
            # up using connect() clauses
            if c.type == 'connector':
                flat_class.symbols[flat_sym.name] = flat_sym

    # now resolve all references inside the symbol definitions
    for sym_name, sym in flat_class.symbols.items():
        flat_sym = flatten_component_refs(root, flat_class, sym, instance_prefix)
        flat_class.symbols[sym_name] = flat_sym

    # for all equations in original class
    flow_connections = OrderedDict()
    for equation in extended_orig_class.equations:
        flat_equation = flatten_component_refs(root, flat_class, equation, instance_prefix)
        if isinstance(equation, ast.ConnectClause):
            # expand connector
            connect_equations = []

            sym_left = root.find_symbol(flat_class, flat_equation.left)
            sym_right = root.find_symbol(flat_class, flat_equation.right)

            try:
                class_left = root.find_class(sym_left.type)
                # noinspection PyUnusedLocal
                class_right = root.find_class(sym_right.type)
            except KeyError:
                logger.warning("Connector class {} or {} not defined.  "
                               "Assuming it to be an elementary type.".format(sym_left.type, sym_right.type))

                connect_equation = ast.Equation(left=flat_equation.left, right=flat_equation.right)
                connect_equations.append(connect_equation)
            else:
                # TODO: Add check about matching inputs and outputs

                flat_class_left = flatten_class(root, class_left, '')

                for connector_variable in flat_class_left.symbols.values():
                    left_name = flat_equation.left.name + CLASS_SEPARATOR + connector_variable.name
                    right_name = flat_equation.right.name + CLASS_SEPARATOR + connector_variable.name
                    left = ast.ComponentRef(name=left_name, indices=flat_equation.left.indices)
                    right = ast.ComponentRef(name=right_name, indices=flat_equation.right.indices)
                    if len(connector_variable.prefixes) == 0 or connector_variable.prefixes[0] in ['input', 'output']:
                        connect_equation = ast.Equation(left=left, right=right)
                        connect_equations.append(connect_equation)
                    elif connector_variable.prefixes == ['flow']:
                        left_repr = repr(left)
                        right_repr = repr(right)

                        left_connected_variables = flow_connections.get(left_repr, OrderedDict())
                        right_connected_variables = flow_connections.get(right_repr, OrderedDict())

                        left_connected_variables.update(right_connected_variables)
                        connected_variables = left_connected_variables
                        connected_variables[left_repr] = left
                        connected_variables[right_repr] = right

                        for connected_variable in connected_variables:
                            flow_connections[connected_variable] = connected_variables
                    else:
                        raise Exception(
                            "Unsupported connector variable prefixes {}".format(connector_variable.prefixes))

            flat_class.equations += connect_equations
        else:
            # flatten equation
            flat_class.equations += [flat_equation]

    flat_class.statements += [flatten_component_refs(root, flat_class, e, instance_prefix) for e in
                              extended_orig_class.statements]

    # add flow equations
    if len(flow_connections) > 0:
        # TODO Flatten first
        logger.warning(
            "Note: Connections between connectors with flow variables "
            "are not supported across levels of the class hierarchy")

    processed = []  # OrderedDict is not hashable, so we cannot use sets.
    for connected_variables in flow_connections.values():
        if connected_variables not in processed:
            operands = list(connected_variables.values())
            expr = ast.Expression(operator='+', operands=operands[-2:])
            for op in reversed(operands[:-2]):
                expr = ast.Expression(operator='+', operands=[op, expr])
            connect_equation = ast.Equation(left=expr, right=ast.Primary(value=0))
            flat_class.equations += [connect_equation]
            processed.append(connected_variables)

    # TODO: Also drag along any functions we need
    # function_set = set()
    # for eq in flat_class.equations + flat_class.statements:
    #     function_set |= pull_functions(eq, instance_prefix)

    # for f in function_set:
    #     if f not in flat_file.classes:
    #         flat_file.classes.update(flatten(root, f, instance_name).classes)

    return flat_class


def modify_class(root: ast.Collection, class_or_sym: Union[ast.Class, ast.Symbol], modification):
    """
    Apply a modification to a class or symbol.
    :param root: root tree for looking up symbols
    :param class_or_sym: class or symbol to modify
    :param modification: modification to apply
    :return: 
    """
    class_or_sym = copy.deepcopy(class_or_sym)
    for argument in modification.arguments:
        if isinstance(argument, ast.ElementModification):
            if argument.component.name in ast.Symbol.ATTRIBUTES:
                setattr(class_or_sym, argument.component.name, argument.modifications[0])
            else:
                s = root.find_symbol(class_or_sym, argument.component)
                for modification in argument.modifications:
                    if isinstance(modification, ast.ClassModification):
                        s.__dict__.update(modify_class(root, s, modification).__dict__)
                    else:
                        s.value = modification
        elif isinstance(argument, ast.ComponentClause):
            for new_sym in argument.symbol_list:
                orig_sym = class_or_sym.symbols[new_sym.name]
                orig_sym.__dict__.update(new_sym.__dict__)
        elif isinstance(argument, ast.ShortClassDefinition):
            for s in class_or_sym.symbols.values():
                if len(s.type.child) == 0 and s.type.name == argument.name:
                    s.type = argument.component
                    # TODO class modifications to short class definition
        else:
            raise Exception('Unsupported class modification argument {}'.format(argument))
    return class_or_sym


def flatten_symbol(s: ast.Symbol, instance_prefix: str) -> ast.Symbol:
    """
    Given a symbols and a prefix performs name mangling
    :param s: Symbol
    :param instance_prefix: Prefix for instance
    :return: flattened symbol
    """
    s_copy = copy.deepcopy(s)
    s_copy.name = instance_prefix + s.name
    if len(instance_prefix) > 0:
        # Strip 'input' and 'output' prefixes from nested symbols.
        strip_keywords = ['input', 'output']
        for strip_keyword in strip_keywords:
            try:
                s_copy.prefixes.remove(strip_keyword)
            except ValueError:
                pass
    return s_copy


class ComponentRefFlattener(TreeListener):
    """
    A listener that flattens references to components and performs name mangling,
    it also locates all symbols and determines which are states (
    one of the equations contains a derivative of the symbol)
    """
    def __init__(self, root: ast.Collection, container: ast.Class, instance_prefix: str):
        self.root = root
        self.container = container
        self.instance_prefix = instance_prefix
        self.depth = 0
        self.cutoff_depth = sys.maxsize

        super(ComponentRefFlattener, self).__init__()

    def enterComponentRef(self, tree: ast.ComponentRef):
        self.depth += 1
        if self.depth > self.cutoff_depth:
            return

        # Compose flatted name
        new_name = self.instance_prefix + tree.name
        c = tree
        while len(c.child) > 0:
            c = c.child[0]
            new_name += CLASS_SEPARATOR + c.name

        # If the flattened name exists in the container, use it.
        # Otherwise, skip this reference.
        try:
            self.root.find_symbol(self.container, ast.ComponentRef(name=new_name))
        except KeyError:
            # The component was not found in the container.  We leave this
            # reference alone.
            self.cutoff_depth = self.depth
        else:
            tree.name = new_name
            c = tree
            while len(c.child) > 0:
                c = c.child[0]
                if len(c.indices) > 0:
                    tree.indices += c.indices
            tree.child = []

    def exitComponentRef(self, tree: ast.ComponentRef):
        self.depth -= 1
        if self.depth < self.cutoff_depth:
            self.cutoff_depth = sys.maxsize

    def exitExpression(self, tree: ast.Expression):
        """
        When exiting an expression, check if it is a derivative, if it is
        put state prefix on symbol
        """
        if tree.operator == 'der':
            s = self.root.find_symbol(self.container, tree.operands[0])
            s.prefixes.append('state')


def flatten_component_refs(
        root: ast.Collection, container: ast.Class,
        expression: ast.Union[ast.ConnectClause, ast.AssignmentStatement, ast.ForStatement, ast.Symbol],
        instance_prefix: str) -> ast.Union[ast.ConnectClause, ast.AssignmentStatement, ast.ForStatement, ast.Symbol]:
    """
    Flattens component refs in a tree
    :param root: root node
    :param container: class
    :param expression: original expression
    :param instance_prefix: prefix for instance
    :return: flattened expression
    """

    expression_copy = copy.deepcopy(expression)

    w = TreeWalker()
    w.walk(ComponentRefFlattener(root, container, instance_prefix), expression_copy)

    return expression_copy


class FunctionPuller(TreeListener):
    """
    Listener to extract functions
    """
    def __init__(self, instance_prefix: str, root, function_set):
        self.instance_prefix = instance_prefix
        self.root = root
        self.function_set = function_set

        super(FunctionPuller, self).__init__()

    def exitExpression(self, tree: ast.Expression):
        if isinstance(tree.operator, ast.ComponentRef) and \
                        tree.operator.name in self.root.classes:
            self.function_set.add(tree.operator.name)


# noinspection PyUnusedLocal
def pull_functions(root: ast.Collection, expression: ast.Expression, instance_prefix: str) -> set:
    """
    TODO: document
    :param root: collection for performing symbol lookup etc.
    :param expression: 
    :param instance_prefix: 
    :return: 
    """

    expression_copy = copy.deepcopy(expression)

    w = TreeWalker()
    function_set = set()
    w.walk(FunctionPuller(instance_prefix, root, function_set), expression_copy)
    return function_set


def flatten(root: ast.Collection, class_name: str) -> ast.File:
    """
    This function takes and flattens it so that all subclasses instances
    are replaced by the their equations and symbols with name mangling
    of the instance name passed.
    :param root: The root of the tree that contains all files with all class definitions
    :param class_name: The class we want to flatten
    :return: flat_file, a File containing the flattened class
    """

    # The within information is needed at the class level when extending
    for f in root.files:
        for c in f.classes.values():
            c.within = f.within

    # flatten class
    flat_class = flatten_class(root, root.find_class(class_name), '')

    # strip connector symbols
    for i, sym in list(flat_class.symbols.items()):
        try:
            # noinspection PyUnusedLocal
            c = root.find_class(sym.type)
        except KeyError:
            pass
        else:
            del flat_class.symbols[i]

    # flat file
    flat_file = ast.File()
    flat_file.classes[class_name] = flat_class

    return flat_file



#!/usr/bin/env python
"""
Modelica AST definitions
"""
from __future__ import print_function, absolute_import, division, print_function, unicode_literals

import copy
import json
from enum import Enum

from typing import List, Union, Dict


class Visibility(Enum):
    PRIVATE = 0, 'private'
    PROTECTED = 1, 'protected'
    PUBLIC = 2, 'public'

    def __new__(cls, value, name):
        member = object.__new__(cls)
        member._value_ = value
        member.fullname = name
        return member

    def __int__(self):
        return self.value

    def __str__(self):
        return self.fullname

    def __lt__(self, other):
        return self.value < other.value


nan = float('nan')

"""
AST Node Type Hierarchy

Root Class
    Class
        Equation
            ComponentRef
            Expression
            Primary
        IfEquation
            Expression
            Equation
        ForEquation
            Expression
            Equation
        ConnectClause
            ComponentRef
        Symbol
"""


class Node(object):
    def __init__(self, **kwargs):
        self.set_args(**kwargs)

    def set_args(self, **kwargs):
        for key in kwargs.keys():
            if key not in self.__dict__.keys():
                raise KeyError('{:s} not valid arg'.format(key))
            self.__dict__[key] = kwargs[key]

    def __repr__(self):
        return json.dumps(self.to_json(self), indent=2, sort_keys=True)

    @classmethod
    def to_json(cls, var):
        if isinstance(var, list):
            res = [cls.to_json(item) for item in var]
        elif isinstance(var, dict):
            res = {key: cls.to_json(var[key]) for key in var.keys()}
        elif isinstance(var, Node):
            res = {key: cls.to_json(var.__dict__[key]) for key in var.__dict__.keys()}
        elif isinstance(var, Visibility):
            res = str(var)
        else:
            res = var
        return res

    __str__ = __repr__


class Primary(Node):
    def __init__(self, **kwargs):
        self.value = None  # type: Union[bool, float, int, str, type(None)]
        super().__init__(**kwargs)


class Array(Node):
    def __init__(self, **kwargs):
        self.values = []  # type: List[Union[Expression, Primary, ComponentRef, Array]]
        super().__init__(**kwargs)


class Slice(Node):
    def __init__(self, **kwargs):
        self.start = Primary(value=0)  # type: Union[Expression, Primary, ComponentRef]
        self.stop = Primary(value=-1)  # type: Union[Expression, Primary, ComponentRef]
        self.step = Primary(value=1)  # type: Union[Expression, Primary, ComponentRef]
        super().__init__(**kwargs)


class ComponentRef(Node):
    def __init__(self, **kwargs):
        self.name = ''  # type: str
        self.indices = []  # type: List[Union[Expression, Slice, Primary, ComponentRef]]
        self.child = []  # type: List[ComponentRef]
        super().__init__(**kwargs)


class Expression(Node):
    def __init__(self, **kwargs):
        self.operator = None  # type: Union[str, ComponentRef]
        self.operands = []  # type: List[Union[Expression, Primary, ComponentRef, Array, IfExpression]]
        super().__init__(**kwargs)


class IfExpression(Node):
    def __init__(self, **kwargs):
        self.conditions = []  # type: List[Union[Expression, Primary, ComponentRef, Array, IfExpression]]
        self.expressions = []  # type: List[Union[Expression, Primary, ComponentRef, Array, IfExpression]]
        super().__init__(**kwargs)


class Equation(Node):
    def __init__(self, **kwargs):
        self.left = None  # type: Union[Expression, Primary, ComponentRef]
        self.right = None  # type: Union[Expression, Primary, ComponentRef]
        self.comment = ''  # type: str
        super().__init__(**kwargs)


class IfEquation(Node):
    def __init__(self, **kwargs):
        self.conditions = []  # type: List[Union[Expression, Primary, ComponentRef]]
        self.equations = []  # type: List[Union[Expression, ForEquation, ConnectClause, IfEquation]]
        self.comment = ''  # type: str
        super().__init__(**kwargs)


class ForIndex(Node):
    def __init__(self, **kwargs):
        self.name = ''  # type: str
        self.expression = None  # type: Union[Expression, Primary, Slice]
        super().__init__(**kwargs)


class ForEquation(Node):
    def __init__(self, **kwargs):
        self.indices = []  # type: List[ForIndex]
        self.equations = []  # type: List[Union[Equation, ForEquation, ConnectClause]]
        self.comment = None  # type: str
        super().__init__(**kwargs)


class ConnectClause(Node):
    def __init__(self, **kwargs):
        self.left = ComponentRef()  # type: ComponentRef
        self.right = ComponentRef()  # type: ComponentRef
        self.comment = ''  # type: str
        super().__init__(**kwargs)


class AssignmentStatement(Node):
    def __init__(self, **kwargs):
        self.left = []  # type: List[ComponentRef]
        self.right = None  # type: Union[Expression, IfExpression, Primary, ComponentRef]
        self.comment = ''  # type: str
        super().__init__(**kwargs)


class IfStatement(Node):
    def __init__(self, **kwargs):
        self.expressions = []  # type: List[Union[Expression, Primary, ComponentRef]]
        self.statements = []  # type: List[Union[AssignmentStatement, ForStatement]]
        self.comment = ''  # type: str
        super().__init__(**kwargs)


class ForStatement(Node):
    def __init__(self, **kwargs):
        self.indices = []  # type: List[ForIndex]
        self.statements = []  # type: List[Union[AssignmentStatement, ForStatement]]
        self.comment = ''  # type: str
        super().__init__(**kwargs)


class Symbol(Node):
    """
    A mathematical variable or state of the model
    """
    ATTRIBUTES = ['min', 'max', 'start', 'fixed', 'nominal']

    def __init__(self, **kwargs):
        self.name = ''  # type: str
        self.type = ComponentRef()  # type: ComponentRef
        self.prefixes = []  # type: List[str]
        self.redeclare = False  # type: bool
        self.final = False  # type: bool
        self.inner = False  # type: bool
        self.outer = False  # type: bool
        self.dimensions = [Primary(value=1)]  # type: List[Union[Expression, Primary, ComponentRef]]
        self.comment = ''  # type: str
        # params start value is 0 by default from Modelica spec
        self.start = Primary(value=0)  # type: Union[Expression, Primary, ComponentRef, Array]
        self.min = Primary(value=None)  # type: Union[Expression, Primary, ComponentRef, Array]
        self.max = Primary(value=None)  # type: Union[Expression, Primary, ComponentRef, Array]
        self.nominal = Primary(value=None)  # type: Union[Expression, Primary, ComponentRef, Array]
        self.value = Primary(value=None)  # type: Union[Expression, Primary, ComponentRef, Array]
        self.fixed = Primary(value=False)  # type: Primary
        self.id = 0  # type: int
        self.order = 0  # type: int
        self.visibility = Visibility.PRIVATE  # type: Visibility
        self.class_modification = None  # type: ClassModification
        super().__init__(**kwargs)


class ComponentClause(Node):
    def __init__(self, **kwargs):
        self.prefixes = []  # type: List[str]
        self.type = ComponentRef()  # type: ComponentRef
        self.dimensions = [Primary(value=1)]  # type: List[Union[Expression, Primary, ComponentRef]]
        self.comment = []  # type: List[str]
        self.symbol_list = []  # type: List[Symbol]
        super().__init__(**kwargs)


class EquationSection(Node):
    def __init__(self, **kwargs):
        self.initial = False  # type: bool
        self.equations = []  # type: List[Union[Equation, ForEquation, ConnectClause]]
        super().__init__(**kwargs)


class AlgorithmSection(Node):
    def __init__(self, **kwargs):
        self.initial = False  # type: bool
        self.statements = []  # type: List[Union[AssignmentStatement, ForStatement]]
        super().__init__(**kwargs)


class ImportAsClause(Node):
    def __init__(self, **kwargs):
        self.component = ComponentRef()  # type: ComponentRef
        self.name = ''  # type: str
        super().__init__(**kwargs)


class ImportFromClause(Node):
    def __init__(self, **kwargs):
        self.component = ComponentRef()  # type: ComponentRef
        self.symbols = []  # type: List[str]
        super().__init__(**kwargs)


class ElementModification(Node):
    # TODO: Check if ComponentRef modifiers are handled correctly. For example,
    # check HomotopicLinear which extends PartialHomotopic with the modifier
    # "H(min = H_b)".
    def __init__(self, **kwargs):
        self.component = ComponentRef()  # type: Union[ComponentRef]
        self.modifications = []  # type: List[Union[Primary, Expression, ClassModification, Array, ComponentRef]]
        super().__init__(**kwargs)


class ShortClassDefinition(Node):
    def __init__(self, **kwargs):
        self.name = ''  # type: str
        self.type = ''  # type: str
        self.component = ComponentRef()  # type: ComponentRef
        self.class_modification = ClassModification()  # type: ClassModification
        super().__init__(**kwargs)


class ElementReplaceable(Node):
    def __init__(self, **kwargs):
        # TODO, add fields ?
        super().__init__(**kwargs)


class ClassModification(Node):
    def __init__(self, **kwargs):
        self.arguments = []  # type: List[Union[ElementModification, ComponentClause, ShortClassDefinition]]
        super().__init__(**kwargs)


class ExtendsClause(Node):
    def __init__(self, **kwargs):
        self.component = None  # type: ComponentRef
        self.class_modification = None  # type: ClassModification
        self.visibility = Visibility.PRIVATE  # type: Visibility
        super().__init__(**kwargs)


class Class(Node):
    def __init__(self, **kwargs):
        self.name = None  # type: str
        self.imports = []  # type: List[Union[ImportAsClause, ImportFromClause]]
        self.extends = []  # type: List[ExtendsClause]
        self.encapsulated = False  # type: bool
        self.partial = False  # type: bool
        self.final = False  # type: bool
        self.type = ''  # type: str
        self.comment = ''  # type: str
        self.symbols = {}  # type: Dict[str, Symbol]
        self.initial_equations = []  # type: List[Union[Equation, ForEquation]]
        self.equations = []  # type: List[Union[Equation, ForEquation, ConnectClause]]
        self.initial_statements = []  # type: List[Union[AssignmentStatement, ForStatement]]
        self.statements = []  # type: List[Union[AssignmentStatement, ForStatement]]
        self.within = []  # type: List[ComponentRef]
        super().__init__(**kwargs)


class File(Node):
    """
    Represents a .mo file for use in pre-processing before flattening to a single class.
    """

    def __init__(self, **kwargs):
        self.within = []  # type: List[ComponentRef]
        self.classes = {}  # type: Dict[str, Class]
        super().__init__(**kwargs)

    def find_class(self, component_ref: ComponentRef) -> Class:
        """
        Find the class that a component is defined in
        :param component_ref: component reference
        :return: the class that contains the ref
        """
        name = component_ref.name
        for c_name in self.classes.keys():
            c = self.classes[c_name]
            if component_ref.name in c.symbols.keys():
                return c
        raise KeyError('symbol {:s} not found'.format(name))

    def find_symbol(self, c: Class, component_ref: ComponentRef) -> Symbol:
        """
        Given a component ref, lookup the symbol in the given class
        :param c: class to look in
        :param component_ref: component reference
        :return: the symbol
        """
        sym = c.symbols[component_ref.name]
        if len(component_ref.child) > 0:
            c = self.find_class(sym.type)
            return self.find_symbol(c, component_ref.child[0])
        else:
            return sym


class Collection(Node):
    """
    A list of modelica files, used in pre-processing packages etc. before flattening
    to a single class.
    """

    def __init__(self, **kwargs):
        self.files = []  # type: List[File]
        super().__init__(**kwargs)

    def extend(self, other):
        self.files.extend(other.files)

    def find_class(self, component_ref: Union[ComponentRef, str], within: list = None):

        if within is None:
            within = []

        if isinstance(component_ref, str):
            assert component_ref.find('.') == -1
            component_ref = ComponentRef(name=component_ref)

        # First we try to the find the file matching the right 'within'
        if not component_ref.child and not within:
            for f in self.files:
                if not f.within:
                    try:
                        return f.classes[component_ref.name]
                    except KeyError:
                        continue

            # Could not find symbol. Assume it is an elementary type
            # TODO: Is this a correct assumption? What if we have an undefined type that is not elementary?
            raise KeyError(component_ref.name)
        else:
            # TODO: Should we move this to a 'get_parent' method in the ComponentRef class
            c_within = copy.deepcopy(component_ref)

            n = c_within
            while n.child[0].child:
                n = n.child[0]
            class_name = n.child[0].name
            n.child = []

            # Merge the within passed in, and the within we split from the
            # class to be looked up
            extended_within = copy.deepcopy(within)

            if extended_within:
                n = extended_within[0]
                while n.child:
                    n = n.child[0]
                n.child.append(c_within)
                extended_within = extended_within[0]
            else:
                extended_within = c_within

            c = next((f.classes[class_name] for f in self.files if
                      f.within and compare_component_ref(f.within[0], extended_within) and class_name in f.classes),
                     None)

            # TODO: This could probably be cleaner if we do nested classes.
            # Then we could traverse up the tree until we found a match,
            # instead of just trying twice (once with, once without prepending
            # the passed-in 'within').
            if c is None:
                # Try again with root node lookup instead of relative
                c = next((f.classes[class_name] for f in self.files if
                          f.within and compare_component_ref(f.within[0], c_within) and class_name in f.classes), None)
                if c is None:
                    # TODO: How long do we traverse? Do we somehow force a stop at Real, Boolean, etc?
                    #       Now a force is stopped on anything in the Modelica library.
                    # FIXME: The "SI" part should be removed when we can handle import statements.
                    if c_within.name in ("Modelica", "SI"):
                        raise KeyError(c_within.name)
                    else:
                        raise Exception("Could not find class {} in {}".format(class_name, c_within))
                else:
                    return c
            else:
                return c

    def find_symbol(self, c: Class, component_ref: ComponentRef) -> Symbol:
        sym = c.symbols[component_ref.name]
        if len(component_ref.child) > 0:
            c = self.find_class(sym.type)
            return self.find_symbol(c, component_ref.child[0])
        else:
            return sym


def compare_component_ref(this: ComponentRef, other: ComponentRef) -> bool:
    """
    Helper function to compare component references to each other without converting to JSON
    :param this: 
    :param other: 
    :return: boolean, true if match
    """
    if len(this.child) != len(other.child):
        return False

    if this.child and other.child:
        return compare_component_ref(this.child[0], other.child[0])

    return this.__dict__ == other.__dict__




# do not edit, generated by pymola

from __future__ import print_function, division
import sympy
import sympy.physics.mechanics as mech
from pymola.sympy_runtime import OdeModel
from sympy import sin, cos, tan


class Aircraft(OdeModel):

    def __init__(self):

        super(Aircraft, self).__init__()

        # states
        body__x, body__v_x, body__a_x = mech.dynamicsymbols('body.x, body.v_x, body.a_x')
        self.x = sympy.Matrix([body__x, body__v_x, body__a_x])
        self.x0 = {
            body__x : 0.0,
            body__v_x : 0.0,
            body__a_x : 0.0,
            }

        # variables
        accel__ma_x, accel__b_x__y = mech.dynamicsymbols('accel.ma_x, accel.b_x.y')
        self.v = sympy.Matrix([accel__ma_x, accel__b_x__y])

        # constants
        self.c = sympy.Matrix([])
        self.c0 = {
            }

        # parameters
        body__g, body__c, body__m, accel__b_x__b = sympy.symbols('body.g, body.c, body.m, accel.b_x.b')
        self.p = sympy.Matrix([body__g, body__c, body__m, accel__b_x__b])
        self.p0 = {
            body__g : 9.81,
            body__c : 0.9,
            body__m : 1.0,
            accel__b_x__b : 0.0,
            }

        # inputs
        body__f_x, accel__a_x, accel__b_x__u = mech.dynamicsymbols('body.f_x, accel.a_x, accel.b_x.u')
        self.u = sympy.Matrix([body__f_x, accel__a_x, accel__b_x__u])
        self.u0 = {
            body__f_x : 0.0,
            accel__a_x : 0.0,
            accel__b_x__u : 0.0,
            }

        # outputs
        body__x, body__v_x, body__a_x, accel__ma_x, accel__b_x__y = mech.dynamicsymbols('body.x, body.v_x, body.a_x, accel.ma_x, accel.b_x.y')
        self.y = sympy.Matrix([body__x, body__v_x, body__a_x, accel__ma_x, accel__b_x__y])

        # equations
        self.eqs = [
            body__a_x - (accel__a_x),
            (body__x).diff(self.t) - (body__v_x),
            (body__v_x).diff(self.t) - (body__a_x),
            body__f_x - (body__m * body__a_x),
            accel__b_x__u - (accel__a_x),
            accel__ma_x - (accel__b_x__y),
            accel__b_x__y - (accel__b_x__u + accel__b_x__b),
            ]

        self.compute_fg()


#!/usr/bin/env python
"""
Modelica parse Tree to AST tree.
"""
from __future__ import print_function, absolute_import, division, print_function, unicode_literals

import os
import sys
import time
import unittest

from pymola import gen_sympy
from pymola import parser
from pymola import tree

TEST_DIR = os.path.dirname(os.path.realpath(__file__))


class GenSympyTest(unittest.TestCase):
    """
    Sympy generation test
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @staticmethod
    def flush():
        sys.stdout.flush()
        sys.stdout.flush()
        time.sleep(0.01)

    def test_estimator(self):
        with open(os.path.join(TEST_DIR, 'Estimator.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        text = gen_sympy.generate(ast_tree, 'Estimator')
        with open(os.path.join(TEST_DIR, 'generated/Estimator.py'), 'w') as f:
            f.write(text)
        from test.generated.Estimator import Estimator as Estimator
        e = Estimator()
        # noinspection PyUnusedLocal
        res = e.simulate(x0=[1.0])
        self.flush()

    def test_spring(self):
        with open(os.path.join(TEST_DIR, 'Spring.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        text = gen_sympy.generate(ast_tree, 'Spring')
        with open(os.path.join(TEST_DIR, 'generated/Spring.py'), 'w') as f:
            f.write(text)
        from test.generated.Spring import Spring as Spring
        e = Spring()
        # noinspection PyUnusedLocal
        res = e.simulate(x0=[1.0, 1.0])
        self.flush()

    def test_aircraft(self):
        with open(os.path.join(TEST_DIR, 'Aircraft.mo'), 'r') as f:
            txt = f.read()
        # noinspection PyUnusedLocal
        ast_tree = parser.parse(txt)
        # text = gen_sympy.generate(ast_tree, 'Aircraft')
        # with open(os.path.join(TEST_DIR, 'generated/Aircraft.py'), 'w') as f:
        #  f.write(text)
        # from generated.Aircraft import Aircraft as Aircraft
        # e = Aircraft()
        # res = e.simulate()
        self.flush()

    @unittest.skip
    def test_connector(self):
        with open(os.path.join(TEST_DIR, 'Connector.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        # print(ast_tree)

        # noinspection PyUnusedLocal
        flat_tree = tree.flatten(ast_tree, 'Aircraft')
        # print(flat_tree)

        # noinspection PyUnusedLocal
        walker = tree.TreeWalker()
        # noinspection PyUnusedLocal
        classes = ast_tree.classes
        # noinspection PyUnusedLocal
        root = ast_tree.classes['Aircraft']

        # instantiator = tree.Instantiator(classes=classes)
        # walker.walk(instantiator, root)
        # print(instantiator.res[root].symbols.keys())
        # print(instantiator.res[root])

        # print('INSTANTIATOR\n-----------\n\n')
        # print(instantiator.res[root])

        # connectExpander = tree.ConnectExpander(classes=classes)
        # walker.walk(connectExpander, instantiator.res[root])

        # print('CONNECT EXPANDER\n-----------\n\n')
        # print(connectExpander.new_class)

        # text = gen_sympy.generate(ast_tree, 'Aircraft')
        # print(text)
        # with open(os.path.join(TEST_DIR, 'generated/Connect.py'), 'w') as f:
        #    f.write(text)

        # from generated.Connect import Aircraft as Aircraft
        # e = Aircraft()
        # res = e.simulate()
        self.flush()


if __name__ == "__main__":
    unittest.main()




# do not edit, generated by pymola

from __future__ import print_function, division
import sympy
import sympy.physics.mechanics as mech
from pymola.sympy_runtime import OdeModel
from sympy import sin, cos, tan


class Estimator(OdeModel):

    def __init__(self):

        super(Estimator, self).__init__()

        # states
        x = mech.dynamicsymbols('x')
        self.x = sympy.Matrix([x])
        self.x0 = {
            x : 0,
            }

        # variables
        y = mech.dynamicsymbols('y')
        self.v = sympy.Matrix([y])

        # constants
        self.c = sympy.Matrix([])
        self.c0 = {
            }

        # parameters
        self.p = sympy.Matrix([])
        self.p0 = {
            }

        # inputs
        self.u = sympy.Matrix([])
        self.u0 = {
            }

        # outputs
        y = mech.dynamicsymbols('y')
        self.y = sympy.Matrix([y])

        # equations
        self.eqs = [
            (x).diff(self.t) - (- x),
            y - (x),
            ]

        self.compute_fg()


#!/usr/bin/env python
"""
Modelica parse Tree to AST tree.
"""
from __future__ import print_function, absolute_import, division, unicode_literals

import os
import unittest

import casadi as ca
import numpy as np

from pymola import gen_casadi
from pymola import parser

CasadiSysModel = gen_casadi.CasadiSysModel
TEST_DIR = os.path.dirname(os.path.realpath(__file__))


# noinspection PyPep8Naming,PyUnresolvedReferences
class GenCasadiTest(unittest.TestCase):
    def assert_model_equivalent(self, A, B):
        def sstr(a): return set([str(e) for e in a])

        for l in ["states", "der_states", "inputs", "outputs", "constants", "parameters", "equations"]:
            self.assertEqual(sstr(getattr(A, l)), sstr(getattr(B, l)))

    def assert_model_equivalent_numeric(self, A, B, tol=1e-9):
        self.assertEqual(len(A.states), len(B.states))
        self.assertEqual(len(A.der_states), len(B.der_states))
        self.assertEqual(len(A.inputs), len(B.inputs))
        self.assertEqual(len(A.outputs), len(B.outputs))
        self.assertEqual(len(A.constants), len(B.constants))
        self.assertEqual(len(A.constant_values), len(B.constant_values))
        self.assertEqual(len(A.parameters), len(B.parameters))
        self.assertEqual(len(A.equations), len(B.equations))

        for a, b in zip(A.constant_values, B.constant_values):
            delta = ca.vec(a - b)
            for i in range(delta.size1()):
                test = float(delta[i]) <= tol
                self.assertTrue(test)

        this = A.dae_residual_function(group_arguments=False)
        that = B.dae_residual_function(group_arguments=False)

        this_mx = this.mx_in()
        that_mx = that.mx_in()
        this_in = [e.name() for e in this_mx if e.is_symbolic()]
        that_in = [e.name() for e in that_mx if e.is_symbolic()]

        that_from_this = []
        this_mx_dict = dict(zip(this_in, this_mx))
        that_mx_dict = dict(zip(that_in, that_mx))
        for e in that_in:
            self.assertTrue(e in this_in)
            self.assertEqual(this_mx_dict[e].size1(), that_mx_dict[e].size1())
            self.assertEqual(this_mx_dict[e].size2(), that_mx_dict[e].size2())
            that_from_this.append(this_mx_dict[e])

        that = ca.Function('f', this_mx, that.call(that_from_this))

        np.random.seed(0)

        args_in = []
        for i in range(this.n_in()):
            sp = this.sparsity_in(0)
            r = ca.DM(sp, np.random.random(sp.nnz()))
            args_in.append(r)

        this_out = this.call(args_in)
        that_out = that.call(args_in)

        for i, (a, b) in enumerate(zip(this_out, that_out)):
            test = float(ca.norm_2(ca.vec(a - b))) <= tol
            if not test:
                print("Expr mismatch")
                print("A: ", A.equations[i], a)
                print("B: ", B.equations[i], b)
            self.assertTrue(test)

        return True

    def test_spring(self):
        with open(os.path.join(TEST_DIR, 'Spring.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'Spring')
        ref_model = CasadiSysModel()
        print(casadi_model)
        x = ca.MX.sym("x")
        v_x = ca.MX.sym("v_x")
        der_x = ca.MX.sym("der(x)")
        der_v_x = ca.MX.sym("der(v_x)")
        k = ca.MX.sym("k")
        c = ca.MX.sym("c")
        ref_model.states = [x, v_x]
        ref_model.der_states = [der_x, der_v_x]
        ref_model.parameters = [c, k]
        ref_model.equations = [der_x - v_x, der_v_x - (-k * x - c * v_x)]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_estimator(self):
        with open(os.path.join(TEST_DIR, 'Estimator.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'Estimator')
        ref_model = CasadiSysModel()
        print(casadi_model)

        x = ca.MX.sym("x")
        der_x = ca.MX.sym("der(x)")
        y = ca.MX.sym("y")

        ref_model.states = [x]
        ref_model.der_states = [der_x]
        ref_model.alg_states = [y]
        ref_model.outputs = [y]
        ref_model.equations = [der_x + x, y - x]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_aircraft(self):
        with open(os.path.join(TEST_DIR, 'Aircraft.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        # noinspection PyUnusedLocal
        casadi_model = gen_casadi.generate(ast_tree, 'Aircraft')
        # noinspection PyUnusedLocal
        ref_model = CasadiSysModel()
        self.assertTrue(True)

    def test_connector(self):
        with open(os.path.join(TEST_DIR, 'ConnectorHQ.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'System')
        ref_model = CasadiSysModel()
        print(casadi_model)

        a__up__H = ca.MX.sym("a.up.H")
        a__up__Q = ca.MX.sym("a.up.Q")
        a__down__H = ca.MX.sym("a.down.H")
        a__down__Q = ca.MX.sym("a.down.Q")

        b__up__H = ca.MX.sym("b.up.H")
        b__up__Q = ca.MX.sym("b.up.Q")
        b__down__H = ca.MX.sym("b.down.H")
        b__down__Q = ca.MX.sym("b.down.Q")

        c__up__H = ca.MX.sym("c.up.H")
        c__up__Q = ca.MX.sym("c.up.Q")
        c__down__H = ca.MX.sym("c.down.H")
        c__down__Q = ca.MX.sym("c.down.Q")

        qa__down__H = ca.MX.sym("qa.down.H")
        qa__down__Q = ca.MX.sym("qa.down.Q")
        qc__down__H = ca.MX.sym("qc.down.H")
        qc__down__Q = ca.MX.sym("qc.down.Q")

        hb__up__H = ca.MX.sym("hb.up.H")
        hb__up__Q = ca.MX.sym("hb.up.Q")

        ref_model.alg_states = [qc__down__H, a__down__H, b__down__H, c__down__H, c__up__H, hb__up__H, a__up__H,
                                b__up__H, qa__down__H, a__up__Q, qa__down__Q, c__down__Q, hb__up__Q, c__up__Q, b__up__Q,
                                b__down__Q, qc__down__Q, a__down__Q]

        ref_model.equations = [a__up__H - a__down__H,
                               a__up__Q + a__down__Q,
                               c__up__H - c__down__H,
                               c__up__Q + c__down__Q,

                               b__up__H - b__down__H,
                               b__up__Q + b__down__Q,

                               qa__down__Q,
                               qc__down__Q,

                               hb__up__H,

                               qa__down__H - a__up__H,
                               qc__down__H - c__up__H,
                               a__down__H - b__up__H,
                               c__down__H - b__up__H,
                               b__down__H - hb__up__H,

                               a__down__Q + (b__up__Q + c__down__Q),
                               qc__down__Q + c__up__Q,
                               b__down__Q + hb__up__Q,
                               qa__down__Q + a__up__Q]

        print(ref_model)
        self.assert_model_equivalent(ref_model, casadi_model)

    def test_connector_hqz(self):
        with open(os.path.join(TEST_DIR, 'ConnectorHQZ.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'SystemZ')
        ref_model = CasadiSysModel()
        print(casadi_model)

        a__up__H = ca.MX.sym("a.up.H")
        a__up__Q = ca.MX.sym("a.up.Q")
        a__up__Z = ca.MX.sym("a.up.Z")
        a__down__H = ca.MX.sym("a.down.H")
        a__down__Q = ca.MX.sym("a.down.Q")
        a__down__Z = ca.MX.sym("a.down.Z")

        b__up__H = ca.MX.sym("b.up.H")
        b__up__Q = ca.MX.sym("b.up.Q")
        b__up__Z = ca.MX.sym("b.up.Z")
        b__down__H = ca.MX.sym("b.down.H")
        b__down__Q = ca.MX.sym("b.down.Q")
        b__down__Z = ca.MX.sym("b.down.Z")

        c__up__H = ca.MX.sym("c.up.H")
        c__up__Q = ca.MX.sym("c.up.Q")
        c__up__Z = ca.MX.sym("c.up.Z")
        c__down__H = ca.MX.sym("c.down.H")
        c__down__Q = ca.MX.sym("c.down.Q")
        c__down__Z = ca.MX.sym("c.down.Z")

        qa__down__H = ca.MX.sym("qa.down.H")
        qa__down__Q = ca.MX.sym("qa.down.Q")
        qa__down__Z = ca.MX.sym("qa.down.Z")
        qc__down__H = ca.MX.sym("qc.down.H")
        qc__down__Q = ca.MX.sym("qc.down.Q")
        qc__down__Z = ca.MX.sym("qc.down.Z")

        hb__up__H = ca.MX.sym("hb.up.H")
        hb__up__Q = ca.MX.sym("hb.up.Q")
        hb__up__Z = ca.MX.sym("hb.up.Z")

        ref_model.alg_states = [qc__down__H, a__down__H, b__down__H, c__down__H, c__up__H, hb__up__H, a__up__H,
                                b__up__H, qa__down__H, a__up__Q, qa__down__Q, c__down__Q, hb__up__Q, c__up__Q, b__up__Q,
                                b__down__Q, qc__down__Q, a__down__Q, a__up__Z, a__down__Z, b__up__Z, b__down__Z,
                                c__up__Z, c__down__Z, qa__down__Z, qc__down__Z, hb__up__Z]

        ref_model.equations = [a__up__H - a__down__H,
                               a__up__Q + a__down__Q,
                               c__up__H - c__down__H,
                               c__up__Q + c__down__Q,

                               b__up__H - b__down__H,
                               b__up__Q + b__down__Q,

                               qa__down__Q,
                               qc__down__Q,

                               hb__up__H,

                               qa__down__H - a__up__H,
                               qc__down__H - c__up__H,
                               a__down__H - b__up__H,
                               c__down__H - b__up__H,
                               b__down__H - hb__up__H,

                               a__down__Q + (b__up__Q + c__down__Q),
                               qc__down__Q + c__up__Q,
                               b__down__Q + hb__up__Q,
                               qa__down__Q + a__up__Q,

                               qa__down__Z - a__up__Z,
                               qc__down__Z - c__up__Z,
                               a__down__Z - b__up__Z,
                               c__down__Z - b__up__Z,
                               b__down__Z - hb__up__Z]

        print(ref_model)
        self.assert_model_equivalent(ref_model, casadi_model)

    def test_duplicate(self):
        with open(os.path.join(TEST_DIR, 'DuplicateState.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'DuplicateState')
        print(casadi_model)
        ref_model = CasadiSysModel()

        x = ca.MX.sym("x")
        der_x = ca.MX.sym("der(x)")
        y = ca.MX.sym("y")
        der_y = ca.MX.sym("der(y)")

        ref_model.states = [x, y]
        ref_model.der_states = [der_x, der_y]
        ref_model.equations = [der_x + der_y - 1, der_x - 2]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_if_else(self):
        with open(os.path.join(TEST_DIR, 'IfElse.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'IfElse')
        ref_model = CasadiSysModel()

        x = ca.MX.sym("x")
        y1 = ca.MX.sym("y1")
        y2 = ca.MX.sym("y2")
        y3 = ca.MX.sym("y3")
        y_max = ca.MX.sym("y_max")

        ref_model.inputs = [x]
        ref_model.outputs = [y1, y2, y3]
        ref_model.alg_states = [x, y1, y2, y3]
        ref_model.parameters = [y_max]
        ref_model.equations = [
            y1 - ca.if_else(x > 0, 1, 0) * y_max,
            ca.if_else(x > 1, ca.vertcat(y3 - 100, y2 - y_max),
                       ca.if_else(x > 2, ca.vertcat(y3 - 1000, y2 - y_max - 1),
                                  ca.vertcat(y3 - 10000, y2)))]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_inheritance(self):
        with open(os.path.join(TEST_DIR, 'Inheritance.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'Sub')
        ref_model = CasadiSysModel()

        x = ca.MX.sym("x")
        der_x = ca.MX.sym("der(x)")
        y = ca.MX.sym("y")
        # noinspection PyUnusedLocal
        der_y = ca.MX.sym("y")
        k = ca.MX.sym("k")

        ref_model.states = [x]
        ref_model.der_states = [der_x]
        ref_model.alg_states = [y]
        ref_model.parameters = [k]
        ref_model.equations = [der_x - k * x, x + y - 3]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_inheritance_instantiation(self):
        with open(os.path.join(TEST_DIR, 'InheritanceInstantiation.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'C2')
        ref_model = CasadiSysModel()
        print(casadi_model)

        bcomp1_a = ca.MX.sym('bcomp1.a')
        bcomp1_b = ca.MX.sym('bcomp1.b')
        bcomp2_a = ca.MX.sym('bcomp2.a')
        bcomp2_b = ca.MX.sym('bcomp2.b')
        bcomp3_a = ca.MX.sym('bcomp3.a')
        bcomp3_b = ca.MX.sym('bcomp3.b')

        bcomp1_v = ca.MX.sym('bcomp1.v', 3)
        bcomp2_v = ca.MX.sym('bcomp2.v', 4)
        bcomp3_v = ca.MX.sym('bcomp3.v', 2)

        ref_model.states = []
        ref_model.der_states = []
        ref_model.alg_states = [bcomp1_v, bcomp2_v, bcomp3_v]
        ref_model.parameters = [bcomp1_a, bcomp1_b, bcomp2_a, bcomp2_b, bcomp3_a, bcomp3_b]
        ref_model.equations = []

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_builtin(self):
        with open(os.path.join(TEST_DIR, 'BuiltinFunctions.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'BuiltinFunctions')
        print("BuiltinFunctions", casadi_model)
        ref_model = CasadiSysModel()

        x = ca.MX.sym("x")
        y = ca.MX.sym("y")
        z = ca.MX.sym("z")
        w = ca.MX.sym("w")
        u = ca.MX.sym("u")

        ref_model.inputs = [x]
        ref_model.outputs = [y, z, w, u]
        ref_model.alg_states = [x, y, z, w, u]
        ref_model.equations = [y - ca.sin(ref_model.time), z - ca.cos(x), w - ca.fmin(y, z), u - ca.fabs(w)]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_forloop(self):
        with open(os.path.join(TEST_DIR, 'ForLoop.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'ForLoop')
        print(casadi_model)
        ref_model = CasadiSysModel()

        x = ca.MX.sym("x", 10)
        y = ca.MX.sym("y", 10)
        z = ca.MX.sym("z", 10)
        w = ca.MX.sym('w', 2, 10)
        b = ca.MX.sym("b")
        n = ca.MX.sym("n")

        ref_model.alg_states = [x, y, z, w, b]
        ref_model.parameters = [n]
        ref_model.equations = [
            ca.horzcat(x - (np.arange(1, 11) + b), w[0, :].T - np.arange(1, 11), w[1, :].T - np.arange(2, 21, 2)),
            y[0:5] - np.zeros(5), y[5:] - np.ones(5),
            ca.horzcat(z[0:5] - np.array([2, 2, 2, 2, 2]), z[5:10] - np.array([1, 1, 1, 1, 1]))]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_arrayexpressions(self):
        with open(os.path.join(TEST_DIR, 'ArrayExpressions.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'ArrayExpressions')
        print(casadi_model)
        ref_model = CasadiSysModel()

        a = ca.MX.sym("a", 3)
        b = ca.MX.sym("b", 4)
        c = ca.MX.sym("c", 3)
        d = ca.MX.sym("d", 3)
        e = ca.MX.sym("e", 3)
        g = ca.MX.sym("g", 1)
        h = ca.MX.sym("h", 1)
        B = ca.MX.sym("B", 3)
        C = ca.MX.sym("C", 2)
        D = ca.MX.sym("D", 3)
        E = ca.MX.sym("E", 2)
        arx = ca.MX.sym("ar.x", 3)
        arcy = ca.MX.sym("arc.y", 2)
        arcw = ca.MX.sym("arc.w", 2)
        nested1z = ca.MX.sym('nested1.z', 3)
        nested2z = ca.MX.sym('nested2.z', 2, 3)
        nested1n = ca.MX.sym('nested1.n', 1)
        nested2n = ca.MX.sym('nested2.n', 2)

        scalar_f = ca.MX.sym("scalar_f")
        c_dim = ca.MX.sym("c_dim")
        d_dim = ca.MX.sym("d_dim")

        ref_model.alg_states = [a, c, d, e, scalar_f, g, arx, arcy, arcw, nested1z, nested2z, h]
        ref_model.parameters = [d_dim, nested1n, nested2n]
        ref_model.outputs = [h]
        ref_model.constants = [b, c_dim, B, C, D, E]
        ref_model.constant_values = [np.array([2.7, 3.7, 4.7, 5.7]), 2, ca.linspace(1, 2, 3), 1.7 * ca.DM.ones(2),
                                     ca.DM.zeros(3), ca.DM.ones(2)]
        ref_model.equations = [c - (a + b[0:3] * e), d - (ca.sin(a / b[1:4])), e - (d + scalar_f), g - ca.sum1(c),
                               h - B[1], arx[1] - scalar_f, nested1z - ca.DM.ones(3), nested2z[0, :].T - ca.DM.zeros(3),
                               nested2z[1, 0] - 3, nested2z[1, 1] - 2, nested2z[1, 2] - 1, arcy[0] - arcy[1],
                               arcw[0] + arcw[1]]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_matrixexpressions(self):
        with open(os.path.join(TEST_DIR, 'MatrixExpressions.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'MatrixExpressions')
        print(casadi_model)
        ref_model = CasadiSysModel()

        A = ca.MX.sym("A", 3, 3)
        b = ca.MX.sym("b", 3)
        c = ca.MX.sym("c", 3)
        d = ca.MX.sym("d", 3)
        C = ca.MX.sym("C", 2, 3)
        D = ca.MX.sym("D", 3, 2)
        E = ca.MX.sym("E", 2, 3)
        I = ca.MX.sym("I", 5, 5)
        F = ca.MX.sym("F", 3, 3)

        ref_model.alg_states = [A, b, c, d]
        ref_model.constants = [C, D, E, I, F]
        ref_model.constant_values = [1.7 * ca.DM.ones(2, 3), ca.DM.zeros(3, 2), ca.DM.ones(2, 3), ca.DM.eye(5),
                                     ca.DM.triplet([0, 1, 2], [0, 1, 2], [1, 2, 3], 3, 3)]
        ref_model.equations = [ca.mtimes(A, b) - c, ca.mtimes(A.T, b) - d, F[1, 2]]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_attributes(self):
        with open(os.path.join(TEST_DIR, 'Attributes.mo'), 'r') as f:
            txt = f.read()
        ast_tree = parser.parse(txt)
        casadi_model = gen_casadi.generate(ast_tree, 'Attributes')
        print(casadi_model)
        ref_model = CasadiSysModel()

        i = ca.MX.sym("int")
        b = ca.MX.sym("bool")
        r = ca.MX.sym("real")
        der_r = ca.MX.sym("der(real)")
        i1 = ca.MX.sym("i1")
        i2 = ca.MX.sym("i2")
        i3 = ca.MX.sym("i3")
        i4 = ca.MX.sym("i4")
        cst = ca.MX.sym("cst")
        prm = ca.MX.sym("prm")
        protected_variable = ca.MX.sym("protected_variable")

        ref_model.states = [r]
        ref_model.der_states = [der_r]
        ref_model.alg_states = [i, b, i1, i2, i3, i4, protected_variable]
        ref_model.inputs = [i1, i2, i3]
        ref_model.outputs = [i4, protected_variable]
        ref_model.constants = [cst]
        ref_model.constant_values = [1]
        ref_model.parameters = [prm]
        ref_model.equations = [i4 - ((i1 + i2) + i3), der_r - (i1 + ca.if_else(b, 1, 0) * i),
                               protected_variable - (i1 + i2)]

        self.assert_model_equivalent_numeric(ref_model, casadi_model)

    def test_type(self):
        with open(os.path.join(TEST_DIR, 'Type.mo'), 'r') as f:
            txt = f.read()
        # noinspection PyUnusedLocal
        ast_tree = parser.parse(txt)
        self.assertTrue(True)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""A python/modelica based simulation environment.

Pymola contains a Python based compiler for the modelica language
and enables interacting with Modelica easily in Python.

"""

from __future__ import print_function
from setuptools import setup

import os
import sys
import subprocess
import pprint
import shutil
import fnmatch

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py
from setuptools.command.build_ext import build_ext
from setuptools import Command

from Cython.Build import cythonize
import versioneer

MAJOR = 0
MINOR = 1
MICRO = 5
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)
DOCLINES = __doc__.split("\n")

CLASSIFIERS = """\
Development Status :: 1 - Planning
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Other
Topic :: Software Development
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Scientific/Engineering :: Mathematics
Topic :: Scientific/Engineering :: Physics
Topic :: Scientific/Engineering :: Visualization
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
Topic :: Software Development :: Code Generators
Topic :: Software Development :: Compilers
Topic :: Software Development :: Embedded Systems
"""


ROOT_DIR = os.path.dirname(os.path.realpath(__file__))

#pylint: disable=no-init, too-few-public-methods


import sys
python_version = '.'.join([str(i) for i in sys.version_info[:3]])
python_version_required = '3.4.0'
if python_version < python_version_required:
    sys.exit("Sorry, only Python >= {:s} is supported".format(python_version_required))

class AntlrBuildCommand(Command):
    """Customized setuptools build command."""
    user_options=[]
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        "Run the build command"
        call_antlr4('Modelica.g4')

def call_antlr4(arg):
    "calls antlr4 on grammar file"
    #pylint: disable=unused-argument, unused-variable
    antlr_path = os.path.join(ROOT_DIR, "java", "antlr-4.7-complete.jar")
    classpath = ".:{:s}:$CLASSPATH".format(antlr_path)
    generated = os.path.join(ROOT_DIR, 'pymola', 'generated')
    cmd = "java -Xmx500M -cp \"{classpath:s}\" org.antlr.v4.Tool {arg:s}" \
            " -o {generated:s} -visitor -Dlanguage=Python3".format(**locals())
    print(cmd)
    proc = subprocess.Popen(cmd.split(), cwd=os.path.join(ROOT_DIR, 'pymola'))
    proc.communicate()
    with open(os.path.join(ROOT_DIR, 'pymola', 'generated', '__init__.py'), 'w') as fid:
        fid.write('')
    for root, dir, files in os.walk(generated):
        for item in fnmatch.filter(files, "Modelica*.py"):
            filename = os.path.join(root, item)
            shutil.move(filename, filename.replace('.py', '.pyx'))

def setup_package():
    """
    Setup the package.
    """
    src_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    old_path = os.getcwd()
    os.chdir(src_path)
    sys.path.insert(0, src_path)

    # Install requirements
    with open('requirements.txt', 'r') as req_file:
        install_reqs = req_file.read().split('\n')
    # pprint.pprint(install_reqs)

    # Disable compiler optimization.  We have to do this, as the default -O3 triggers a bug in clang causing an initialization failure.
    #os.environ['CFLAGS'] = '-O0'
    # Or alternatively, use gcc:
    
    os.environ['CC'] = 'gcc'

    # Without disabling this, it will reach a limit
    # on tracking and disable tracking and then recompile, which is slow
    os.environ['CFLAGS'] = '-fno-var-tracking-assignments'

    metadata = dict(
        version=versioneer.get_version(),
        name='pymola',
        maintainer="James Goppert",
        maintainer_email="james.goppert@gmail.com",
        description=DOCLINES[0],
        long_description="\n".join(DOCLINES[2:]),
        url='https://github.com/jgoppert/pymola',
        author='James Goppert',
        author_email='james.goppert@gmail.com',
        download_url='https://github.com/jgoppert/pymola',
        license='BSD',
        classifiers=[_f for _f in CLASSIFIERS.split('\n') if _f],
        platforms=["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
        install_requires=install_reqs,
        tests_require=['coverage >= 3.7.1', 'nose >= 1.3.1'],
        test_suite='nose.collector',
        packages=find_packages(
            # choosing to distribute tests
            # exclude=['*.test']
        ),
        cmdclass={
            'antlr': AntlrBuildCommand,
            'versioneer': versioneer.get_cmdclass(),  
        },
        ext_modules=cythonize('pymola/generated/*.pyx',
            compiler_directives={'boundscheck': False,
                'initializedcheck': False, 'language_level': 3})
    )

    try:
        setup(**metadata)
    finally:
        del sys.path[0]
        os.chdir(old_path)
    return

if __name__ == '__main__':
    setup_package()




# do not edit, generated by pymola

from __future__ import print_function, division
import sympy
import sympy.physics.mechanics as mech
from pymola.sympy_runtime import OdeModel
from sympy import sin, cos, tan

class Quad(OdeModel):

    def __init__(self):

        super(Quad, self).__init__()

        # states
        x, y, z, R, Q, U, R, P, V, Q, P, W, phi, theta, psi, P, Q, R = mech.dynamicsymbols('x, y, z, R, Q, U, R, P, V, Q, P, W, phi, theta, psi, P, Q, R')
        self.x = sympy.Matrix([x, y, z, R, Q, U, R, P, V, Q, P, W, phi, theta, psi, P, Q, R])

        # inputs
        self.u = sympy.Matrix([])

        # outputs
        self.y = sympy.Matrix([])

        # constants
        self.c = sympy.Matrix([])
        self.c0 = {
            }

        # parameters
        J_x, J_y, J_z, m = sympy.symbols('J_x, J_y, J_z, m')
        self.p = sympy.Matrix([J_x, J_y, J_z, m])
        self.p0 = {
            'J_x' : 1,
            'J_y' : 1,
            'J_z' : 1,
            'm' : 1,
            }

        # variables
        F_y, F_z, F_x, M_z, M_y, M_x = sympy.symbols('F_y, F_z, F_x, M_z, M_y, M_x')
        self.v = sympy.Matrix([F_y, F_z, F_x, M_z, M_y, M_x])
      
        # equations
        self.eqs = [
            M_x - (- P - phi),
            M_y - (- Q - theta),
            M_z - (- R - psi),
            F_x - (- x),
            F_y - (- y),
            F_z - (- z),
            (x).diff(self.t) - (U),
            (y).diff(self.t) - (V),
            (z).diff(self.t) - (W),
            - m * V * (R).diff(self.t) + m * W * (Q).diff(self.t) + m * (U).diff(self.t) - (F_x),
            m * U * (R).diff(self.t) - m * W * (P).diff(self.t) + m * (V).diff(self.t) - (F_y),
            - m * U * (Q).diff(self.t) + m * V * (P).diff(self.t) + m * (W).diff(self.t) - (F_z),
            (phi).diff(self.t) - (P + Q * sin(phi) * tan(theta) + R * cos(phi) * tan(theta)),
            (theta).diff(self.t) - (Q * cos(phi) - R * sin(phi)),
            cos(theta) * (psi).diff(self.t) - (Q * sin(phi) + R * cos(phi)),
            (P).diff(self.t) - (M_x),
            (Q).diff(self.t) - (M_y),
            (R).diff(self.t) - (M_z),
            ]

        self.compute_fg()
    


# do not edit, generated by pymola

from __future__ import print_function, division

import pylab as pl
import scipy.integrate
import sympy
from typing import List


# noinspection PyPep8Naming
class OdeModel(object):
    def __init__(self):
        self.t = sympy.symbols('t')
        self.x = sympy.Matrix([])
        self.u = sympy.Matrix([])
        self.y = sympy.Matrix([])
        self.p = sympy.Matrix([])
        self.c = sympy.Matrix([])
        self.v = sympy.Matrix([])
        self.x0 = {}
        self.u0 = {}
        self.p0 = {}
        self.c0 = {}
        self.eqs = []
        self.f = None
        self.g = None

    def compute_fg(self):
        fg_sol = sympy.solve(self.eqs, list(self.x.diff(self.t)) + list(self.v))
        self.f = self.x.diff(self.t).subs(fg_sol)
        assert (len(self.x) == len(self.f))
        self.g = self.y.subs(fg_sol)
        assert (len(self.g) == len(self.y))

    def linearize_symbolic(self):
        A = sympy.Matrix([])
        B = sympy.Matrix([])
        C = sympy.Matrix([])
        D = sympy.Matrix([])
        if len(self.x) > 0:
            if len(self.f) > 0:
                A = self.f.jacobian(self.x)
            if len(self.g) > 0:
                C = self.g.jacobian(self.x)
        if len(self.u) > 0:
            if len(self.f) > 0:
                B = self.f.jacobian(self.u)
            if len(self.g) > 0:
                D = self.g.jacobian(self.u)
        return A, B, C, D

    def linearize(self, x0=None, u0=None):
        ss = self.linearize_symbolic()
        ss_eval = []
        ss_subs = {}
        ss_subs.update(self.p0)
        ss_subs.update(self.c0)
        if x0 is None:
            # noinspection PyUnusedLocal
            x0 = self.x.subs(self.x0)[:]
        if u0 is None:
            # noinspection PyUnusedLocal
            u0 = self.x.subs(self.u0)[:]
        for i in range(len(ss)):
            ss_eval += [pl.matrix(ss[i].subs(ss_subs)).astype(float)]
        return ss_eval

    def simulate(self, x0: List[float]=None, u0: float=None, t0: float=0, tf: float=10, dt: float=0.01) -> dict:
        x_sym = sympy.DeferredVector('x')
        # noinspection PyUnusedLocal
        y_sym = sympy.DeferredVector('y')
        u_sym = sympy.DeferredVector('u')
        ss_subs = {self.x[i]: x_sym[i] for i in range(len(self.x))}
        ss_subs.update({self.u[i]: u_sym[i] for i in range(len(self.u))})
        ss_subs.update(self.p0)
        ss_subs.update(self.c0)

        # create f (dynamics) lambda function
        f_lam = sympy.lambdify((self.t, x_sym, u_sym), self.f.subs(ss_subs))
        res = pl.array(f_lam(0, pl.zeros(len(self.x)), pl.zeros(len(self.u))), dtype=float)
        if len(res) != len(self.x):
            raise IOError("f doesn't return correct size", res, self.x)

        # create jacobian lambda function
        if len(self.x) > 0 and len(self.f) > 0:
            jac_lam = sympy.lambdify((self.t, x_sym, u_sym), self.f.jacobian(self.x).subs(ss_subs))
            res = pl.array(jac_lam(0, pl.zeros(len(self.x)), pl.zeros(len(self.u))), dtype=float)
            if len(res.shape) == 2 and res.shape[0] != len(self.x):
                raise IOError("jacobian doesn't return correct size", res['f'])
        else:
            jac_lam = None

        # create g (measurement) lambda function
        g_lam = sympy.lambdify((self.t, x_sym, u_sym), self.g.subs(ss_subs))
        res = pl.array(g_lam(0, pl.zeros(len(self.x)), pl.zeros(len(self.u))), dtype=float)
        if len(res.shape) == 2 and res.shape[0] != len(self.y):
            raise IOError("g doesn't return correct size", res, self.y)

        ode = scipy.integrate.ode(f_lam, jac_lam)
        ss_subs.update(self.x0)
        ss_subs.update(self.u0)
        if x0 is None:
            x0 = self.x.subs(self.x0)[:]
        if u0 is None:
            u0 = self.u.subs(self.u0)[:]
        ode.set_initial_value(x0, t0)
        y0 = g_lam(0, x0, u0)
        data = {
            't': [t0],
            'x': [x0],
            'y': [y0],
            'u': [u0],
        }
        while ode.t + dt <= tf:
            ode.set_f_params(u0)
            ode.set_jac_params(u0)
            if len(self.x) > 0:
                ode.integrate(ode.t + dt)
            else:
                ode.t += dt
            x = ode.y
            y = g_lam(ode.t, x, u0)
            data['t'] += [ode.t]
            data['x'] += [x]
            data['y'] += [y]
            data['u'] += [u0]
        data = {key: pl.array(data[key]) for key in data.keys()}
        return data

    def __repr__(self):
        return repr(self.__dict__)
